/*
 * licy.js v0.1.0 http://mantoni.github.com/licy.js
 *
 * @generator nomo.js v0.1.5
 */
(function (modules) {
  'use strict';
  var instances = {};
  var delegate = window.require;

  function require(name) {
    if (!instances.hasOwnProperty(name)) {
      var factory = modules[name];
      if (!factory) {
        if (delegate) {
          return instances[name] = { exports: delegate(name) };
        }
        throw new Error('Unknown module: ' + name);
      }
      var exports = {};
      instances[name] = { exports: exports };
      factory(instances[name], exports, require);
    }
    return instances[name].exports;
  }

  require.define = function (name, factory) {
    if (modules[name]) {
      throw new Error('Module "' + name + '" already defined.');
    }
    modules[name] = factory;
  };

  window.licy = require('lib/licy');

}({
  'lib/licy': function (module, exports, require) {
    /**
     * licy.js
     *
     * Copyright (c) 2012 Maximilian Antoni <mail@maxantoni.de>
     *
     * @license MIT
     */
    'use strict';
    function typeOf(value) {
      if (value === null) {
        return 'null';
      }
      var type = Object.prototype.toString.call(value);
      return type.substring(8, type.length - 1).toLowerCase();
    }

    function assertType(value, expectation, name) {
      var type = typeOf(value);
      if (type !== expectation) {
        throw new TypeError('Expected ' + name + ' to be ' + expectation +
          ', but it was ' + type);
      }
    }

    function err(name, message) {
      throw new Error('Plugin "' + name + '" ' + message);
    }

    function dependencyInstaller(deps) {
      return function (raw) {
        var i, l = raw.length;
        for (i = 0; i < l; i++) {
          deps[raw[i][0]] = raw[i][1];
        }
      };
    }


    var hubjs = require('node_modules/hubjs/lib/hub');
    var licy  = hubjs();


    function registerPlugin(name, config) {
      var status        = 'configured';
      var statusEvent   = name + '.status';
      var requireEvent  = name + '.require';
      var startEvent    = name + '.start';
      var stopEvent     = name + '.stop';
      var destroyEvent  = name + '.destroy';
      var instance;

      licy.on(statusEvent, function () {
        return status;
      });

      licy.on(requireEvent, function () {
        if (status === 'started') {
          return [name, instance];
        }
        var callback = this.callback();
        licy.emit(startEvent, function (err) {
          callback(err, [name, instance]);
        });
      });

      licy.before(startEvent, function () {
        if (status === 'starting' || status === 'started') {
          err(name, 'already started');
        }
        status = 'starting';
        var dependencies = config.dependencies;
        if (dependencies) {
          var deps      = this.dependencies = {};
          var strategy  = dependencyInstaller(deps);
          var i, l = dependencies.length;
          for (i = 0; i < l; i++) {
            licy.emit(dependencies[i] + '.require', strategy, this.callback());
          }
        }
      });
      licy.on(startEvent, config.start);
      licy.after(startEvent, function (err, value) {
        instance  = value;
        status    = 'started';
      });

      licy.before(stopEvent, function () {
        if (status === 'stopped') {
          err(name, 'already stopped');
        }
        if (status !== 'started') {
          err(name, 'is not running');
        }
        status = 'stopping';
        this.instance = instance;
      });
      if (config.stop) {
        licy.on(stopEvent, config.stop);
      }
      licy.after(stopEvent, function () {
        status    = 'stopped';
        instance  = null;
      });

      licy.before(destroyEvent, function () {
        if (status === 'started') {
          licy.emit(stopEvent, this.callback());
        }
        status = 'destroying';
      });
      if (config.destroy) {
        licy.on(destroyEvent, config.destroy);
      }
      licy.after(destroyEvent, function () {
        licy.removeAllListeners(statusEvent);
        licy.removeAllListeners(requireEvent);
        licy.removeAllListeners(startEvent);
        licy.removeAllListeners(stopEvent);
        licy.removeAllListeners(destroyEvent);
      });
    }


    var PLUGIN_NOT_ENOUGH_ARGS_ERRORS = [
      'No arguments given.',
      'No config given.'
    ];

    licy.plugin = function (name, config) {
      if (arguments.length < 2) {
        throw new TypeError(PLUGIN_NOT_ENOUGH_ARGS_ERRORS[arguments.length]);
      }
      assertType(name,          'string',   'name');
      assertType(config,        'object',   'config');
      assertType(config.start,  'function', 'config.start');
      registerPlugin(name, config);
    };


    var DEFAULT_CALLBACK = function (err) {
      if (err) {
        throw err;
      }
    };

    licy.start = function (name, callback) {
      assertType(name, 'string', 'name');
      var strategy = name.indexOf('*') === -1 ? hubjs.LAST : hubjs.CONCAT;
      licy.emit(name + '.start', strategy, callback || DEFAULT_CALLBACK);
    };


    licy.stop = function (name, callback) {
      assertType(name, 'string', 'name');
      licy.emit(name + '.stop', callback);
    };


    licy.status = function (name) {
      assertType(name, 'string', 'name');
      var status;
      licy.emit(name + '.status', function (err, value) {
        status = value;
      });
      return status || 'unknown';
    };


    licy.destroy = function (name) {
      assertType(name, 'string', 'name');
      licy.emit(name + '.destroy');
    };

    module.exports = licy;

  },
  'node_modules/hubjs/lib/hub': function (module, exports, require) {
    /**
     * hub.js
     *
     * Copyright (c) 2012 Maximilian Antoni <mail@maxantoni.de>
     *
     * @license MIT
     */
    'use strict';
    var listen = require('node_modules/hubjs/node_modules/listen/lib/listen');


    var DEFAULT_STRATEGY  = function (arr) { return arr[arr.length - 1]; };

    function emit(listeners, scope, args, values, callback) {
      if (listeners) {
        var listener  = listen(values),
          fns         = listeners.slice(),
          l           = listeners.length,
          i;
        scope.callback = listener;

        for (i = 0; i < l; i++) {
          var fn    = fns[i];
          var async = fn.length > args.length;
          if (async) {
            args[fn.length - 1] = listener();
          }
          var value;
          try {
            value = fn.apply(scope, args);
          } catch (e) {
            if (async) {
              args[fn.length - 1](e);
            } else {
              listener.err(e);
            }
          }
          if (value !== undefined) {
            if (async) {
              args.pop()(null, value);
            } else {
              listener.push(value);
            }
          } else if (async) {
            args.pop();
          }
        }

        listener.then(function (err, values) {
          callback.call(scope, err, values);
        });
      } else {
        callback.call(scope, null, values);
      }
    }


    function eventRE(event) {
      return new RegExp('^' + event.replace(/\./g, '\\.').
        replace(/\*\*/g, '[%\\.]+').replace(/\*/g, '[%]+').
        replace(/%/g, 'a-zA-Z_0-9\\-\\*') + '$');
    }


    function invoke(fns, scope, args, values, strategyFn, callback) {
      emit(fns.shift(), scope, args, values, function (err, values) {
        if (err) {
          callback(err);
        } else if (scope.stopped() || !fns.length) {
          callback(null, strategyFn(values));
        } else {
          invoke(fns, scope, args, values, strategyFn, callback);
        }
      });
    }


    function done(scope, callback, err, result) {
      if (callback) {
        callback.call(scope, err, result);
      } else if (err) {
        throw err;
      }
    }


    function indexOf(array, object) {
      var i, l = array.length;
      for (i = 0; i < l; i++) {
        if (array[i] === object) {
          return i;
        }
      }
      return -1;
    }


    function remove(array, object) {
      var i = indexOf(array, object);
      if (i !== -1) {
        array.splice(i, 1);
      }
    }


    function noop() {}


    function create() {
      var listeners = {};
      var matchers  = [];

      function listener(event) {
        var entry = listeners[event];
        if (entry) {
          return entry;
        }
        return (listeners[event] = { before : [], on : [], after : [] });
      }

      function matcher(event) {
        var i, l = matchers.length, entry;
        for (i = 0; i < l; i++) {
          entry = matchers[i];
          if (entry.event === event) {
            return entry;
          }
        }
        var cmp = event.replace(/[a-zA-Z_0-9\-]+/g, '0')
                        .replace(/\./g, '1')
                        .replace(/\*/g, '2');
        entry = {
          event   : event,
          cmp     : cmp,
          re      : eventRE(event),
          before  : [],
          on      : [],
          after   : []
        };
        for (i = 0, l = matchers.length; i < l; i++) {
          if (cmp > matchers[i].cmp) {
            matchers.splice(i, 0, entry);
            return entry;
          }
        }
        matchers.push(entry);
        return entry;
      }

      function typeOf(value) {
        if (value === null) {
          return 'null';
        }
        var type = Object.prototype.toString.call(value);
        return type.substring(8, type.length - 1).toLowerCase();
      }

      function assertType(value, expectation, name) {
        var type = typeOf(value);
        if (type !== expectation) {
          throw new TypeError('Expected ' + name + ' to be ' + expectation +
            ', but it was ' + type);
        }
      }

      var NOT_ENOUGH_ARGS_ERRORS = [
        'No arguments given.',
        'No listener function given.'
      ];

      function asserted(delegate) {
        return function (event, fn) {
          if (arguments.length < 2) {
            throw new TypeError(NOT_ENOUGH_ARGS_ERRORS[arguments.length]);
          }
          assertType(event, 'string',   'event');
          assertType(fn,    'function', 'listener');
          delegate.call(this, event, fn);
        };
      }

      function register(type) {
        return asserted(function (event, fn) {
          this.emit('newListener', event, fn, noop, function () {
            if (!this.stopped()) {
              (event.indexOf('*') === -1 ? listener :
                  matcher)(event)[type].push(fn);
            }
          });
        });
      }

      var removeListener = asserted(function (event, fn) {
        var entry;
        if (event.indexOf('*') === -1) {
          entry = listener(event);
        } else {
          entry = matcher(event);
        }
        remove(entry.on,      fn);
        remove(entry.before,  fn);
        remove(entry.after,   fn);
      });

      var addListener = register('on');

      return {

        before              : register('before'),
        on                  : addListener,
        addListener         : addListener,
        after               : register('after'),
        un                  : removeListener,
        removeListener      : removeListener,

        removeAllListeners  : function (event) {
          if (event) {
            if (event.indexOf('*') === -1) {
              delete listeners[event];
            } else {
              var entry = matcher(event);
              entry.on.length = 0;
              entry.before.length = 0;
              entry.after.length = 0;
            }
          } else {
            listeners = {};
            matchers.length = 0;
          }
        },

        once : function (event, fn) {
          if (arguments.length < 2) {
            throw new TypeError(NOT_ENOUGH_ARGS_ERRORS[arguments.length]);
          }
          var self = this;
          function once() {
            self.un(event, once);
            self.un(event, fn);
          }
          listener(event).on.push(once);
          this.on(event, fn);
        },

        emit : function (event) {
          if (arguments.length === 0) {
            throw new TypeError(NOT_ENOUGH_ARGS_ERRORS[0]);
          }
          var args      = Array.prototype.slice.call(arguments, 1),
            strategyFn  = DEFAULT_STRATEGY,
            broadcast   = event.indexOf('*') !== -1,
            fns         = [],
            fns2        = [],
            matcherBeforeFns,
            matcherAfterFns,
            matcherOnFns,
            callback,
            entry,
            evt,
            re,
            i,
            l;

          if (typeof args[args.length - 1] === 'function') {
            callback = args.pop();
            if (typeof args[args.length - 1] === 'function') {
              strategyFn = args.pop();
            }
          }

          if (broadcast) {
            re = eventRE(event);
            var beforeFns = [];
            var onFns     = [];
            var afterFns  = [];
            for (evt in listeners) {
              if (listeners.hasOwnProperty(evt) && re.test(evt)) {
                entry = listeners[evt];
                beforeFns.push.apply(beforeFns, entry.before);
                onFns.push.apply(onFns, entry.on);
                afterFns.push.apply(afterFns, entry.after);
              }
            }
            fns.push(beforeFns);
            fns.push(onFns);
            fns2.push(afterFns);
          } else {
            entry = listeners[event];
            if (entry) {
              if (entry.before.length) {
                fns.push(entry.before);
              }
              if (entry.on.length) {
                fns.push(entry.on);
              }
              if (entry.after.length) {
                fns2.push(entry.after);
              }
            }
          }

          for (i = 0, l = matchers.length; i < l; i++) {
            var matcher = matchers[i];
            if (matcher.re.test(event) || (re && re.test(matcher.event))) {
              if (!matcherOnFns) {
                matcherBeforeFns  = [];
                matcherAfterFns   = [];
                matcherOnFns      = [];
              }
              matcherBeforeFns.push.apply(matcherBeforeFns, matcher.before);
              matcherOnFns.push.apply(matcherOnFns, matcher.on);
              matcherAfterFns.push.apply(matcherAfterFns, matcher.after);
            }
          }
          if (matcherOnFns) {
            if (matcherOnFns.length) {
              fns.unshift(matcherOnFns);
            }
            if (matcherBeforeFns.length) {
              fns.unshift(matcherBeforeFns);
            }
            if (matcherAfterFns.length) {
              fns2.push(matcherAfterFns);
            }
          }

          var stopped = false;
          var scope   = {
            hub       : this,
            event     : event,
            args      : function () { return args.slice(); },
            stop      : function () { stopped = true; },
            stopped   : function () { return stopped; }
          };

          invoke(fns, scope, args, [], strategyFn, function (err, result) {
            if (stopped) {
              done(scope, callback, err, result);
            } else {
              invoke(fns2, scope, [err, result], [], noop, function () {
                done(scope, callback, err, result);
              });
            }
          });
        }

      };
    }


    var hub = function (listeners) {
      /*jslint forin: true*/
      var instance = create(), event;
      if (listeners) {
        for (event in listeners) {
          var listener = listeners[event];
          if (typeof listener === 'function') {
            instance.on(event, listener);
          }
        }
      }
      return instance;
    };

    hub.listen  = listen;
    hub.LAST    = DEFAULT_STRATEGY;
    hub.CONCAT  = function (arr) { return arr; };

    module.exports = hub;

  },
  'node_modules/hubjs/node_modules/listen/lib/listen': function (module, exports, require) {
    /**
     * listen.js
     *
     * Copyright (c) 2012 Maximilian Antoni <mail@maxantoni.de>
     *
     * @license MIT
     */
    'use strict';
    var ErrorList = require('node_modules/hubjs/node_modules/listen/lib/error-list');

    function listen(initialValues) {
      var values;
      if (arguments.length) {
        if (Object.prototype.toString.call(initialValues) !== '[object Array]') {
          throw new TypeError('Array expected');
        }
        values = Array.prototype.slice.call(initialValues);
      } else {
        values = [];
      }

      var offset  = values.length;
      var count   = 0;
      var called  = 0;
      var errList;
      var handler;

      function resolve(fn) {
        if (errList) {
          fn(errList.length === 1 ? errList[0] : new ErrorList(errList));
        } else {
          fn(null, values);
        }
      }

      function assertUnresolved() {
        if (handler) {
          throw new Error("Cannot be called after then");
        }
      }

      function pushError(e) {
        if (!errList) {
          errList = [];
        }
        if (e.name === 'ErrorList') {
          Array.prototype.push.apply(errList, e.errors);
        } else {
          errList.push(e);
        }
      }

      function listener() {
        assertUnresolved();
        var index = offset + count++;
        return function (err, value) {
          if (err) {
            pushError(err);
          }
          if (value !== undefined) {
            values[index] = value;
          }
          if (++called === count && handler) {
            resolve(handler);
          }
        };
      }

      listener.push = function push(value) {
        assertUnresolved();
        values[offset + count] = value;
        offset++;
      };

      listener.err = function err(e) {
        assertUnresolved();
        pushError(e);
      };

      listener.then = function then(fn) {
        if (handler) {
          throw new Error('Cannot be called more than once');
        }
        if (typeof fn !== 'function') {
          throw new TypeError('Function expected');
        }
        handler = fn;
        if (called === count) {
          resolve(fn);
        }
      };

      return listener;
    }

    listen.ErrorList = ErrorList;

    module.exports = listen;

  },
  'node_modules/hubjs/node_modules/listen/lib/error-list': function (module, exports, require) {
    /**
     * listen.js
     *
     * Copyright (c) 2012 Maximilian Antoni <mail@maxantoni.de>
     *
     * @license MIT
     */
    'use strict';
    function ErrorList(errors) {
      this.name     = 'ErrorList';
      var sep       = '\n  - ';
      this.message  = 'Multiple callbacks err\'d:' + sep + errors.join(sep);
      this.errors   = errors;
    }

    ErrorList.prototype = Error.prototype;

    module.exports = ErrorList;

  }
}));
